@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,17 +53,17 @@ import java.io.IOException;
 import java.io.Serializable;
 import java.io.UnsupportedEncodingException;
 import java.lang.management.ManagementFactory;
+import java.lang.management.ThreadMXBean;
 import java.net.InetAddress;
 import java.net.MalformedURLException;
 import java.net.ServerSocket;
-import java.net.Socket;
 import java.net.UnknownHostException;
 import java.nio.ByteBuffer;
 import java.nio.charset.Charset;
 import java.rmi.RemoteException;
 import java.rmi.registry.LocateRegistry;
 import java.rmi.registry.Registry;
-import java.rmi.server.RMISocketFactory;
+import java.rmi.server.RMIServerSocketFactory;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.util.Collections;
@@ -93,14 +93,14 @@ import com.openexchange.management.ManagementException;
 
 /**
  * {@link AbstractAgent} - An abstract JMX agent
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public abstract class AbstractAgent {
 
-    static final Log LOG = LogFactory.getLog(AbstractAgent.class);
+    static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(AbstractAgent.class));
 
-    private static final class AbstractAgentSocketFactory extends RMISocketFactory implements Serializable {
+    private static final class AbstractAgentSocketFactory implements RMIServerSocketFactory, Serializable {
 
         /**
          * serialVersionUID
@@ -112,23 +112,34 @@ public abstract class AbstractAgent {
         private final InetAddress bindAddress;
 
         public AbstractAgentSocketFactory(final int backlog, final String bindAddr) throws UnknownHostException {
-            this.backlog = backlog < 1 ? 50 : backlog;
+            this.backlog = backlog < 1 ? 0 : backlog;
             bindAddress = bindAddr.charAt(0) == '*' ? null : InetAddress.getByName(bindAddr);
         }
 
-        @Override
         public ServerSocket createServerSocket(final int port) throws IOException {
             return new ServerSocket(port, backlog, bindAddress);
         }
 
         @Override
-        public Socket createSocket(final String host, final int port) throws IOException {
-            return new Socket(bindAddress == null ? InetAddress.getByName(host) : bindAddress, port);
+        public boolean equals(final Object obj) {
+            if (obj == null) {
+                return false;
+            }
+            if (obj == this) {
+                return true;
+            }
+
+            return obj.getClass().equals(getClass());
+        }
+
+        @Override
+        public int hashCode() {
+            return AbstractAgentSocketFactory.class.hashCode();
         }
 
     }
 
-    private static final class AbstractAgentJMXAuthenticator implements JMXAuthenticator {
+    protected static final class AbstractAgentJMXAuthenticator implements JMXAuthenticator {
 
         private static volatile Charset US_ASCII;
 
@@ -175,8 +186,7 @@ public abstract class AbstractAgent {
         }
 
         private static String makeSHAPasswd(final String raw) {
-            MessageDigest md;
-
+            final MessageDigest md;
             try {
                 md = MessageDigest.getInstance("SHA-1");
             } catch (final NoSuchAlgorithmException e) {
@@ -204,15 +214,17 @@ public abstract class AbstractAgent {
 
     }
 
-    private final AtomicBoolean initialized;
+    protected final AtomicBoolean initialized;
 
-    private final Map<Integer, Registry> registries;
+    protected final Map<Integer, Registry> registries;
 
-    private final Map<JMXServiceURL, JMXConnectorServer> connectors;
+    protected final Map<JMXServiceURL, JMXConnectorServer> connectors;
+
+    protected final ThreadMXBean threadMXBean;
 
     protected final AtomicReference<GaugeMonitor> gaugeMonitorRef;
 
-    protected RMISocketFactory rmiSocketFactory;
+    protected RMIServerSocketFactory rmiSocketFactory;
 
     protected MBeanServer mbs;
 
@@ -225,12 +237,22 @@ public abstract class AbstractAgent {
         initialized = new AtomicBoolean();
         registries = new HashMap<Integer, Registry>();
         connectors = new HashMap<JMXServiceURL, JMXConnectorServer>();
+        threadMXBean = ManagementFactory.getThreadMXBean();
         mbs = ManagementFactory.getPlatformMBeanServer();
     }
 
     /**
-     * Uniquely identifies the MBean and registers it to MBeanServer
+     * Gets the {@link ThreadMXBean} instance.
      * 
+     * @return The {@link ThreadMXBean} instance
+     */
+    public ThreadMXBean getThreadMXBean() {
+        return threadMXBean;
+    }
+
+    /**
+     * Uniquely identifies the MBean and registers it to MBeanServer
+     *
      * @param name The MBeans's object name as a string
      * @param mbean The MBean object
      * @throws ManagementException If registering the MBean to MBeanServer fails
@@ -245,7 +267,7 @@ public abstract class AbstractAgent {
 
     /**
      * Unregister the MBean identified through specified object name as a string.
-     * 
+     *
      * @param name The MBean's object name as a string
      * @throws ManagementException If unregistering the MBean fails
      */
@@ -259,7 +281,7 @@ public abstract class AbstractAgent {
 
     /**
      * Uniquely identifies the MBean and registers it to MBeanServer
-     * 
+     *
      * @param objectName The MBean's object name
      * @param mbean The MBean object
      * @throws ManagementException If registering the MBean to MBeanServer fails
@@ -283,7 +305,7 @@ public abstract class AbstractAgent {
 
     /**
      * Unregister the MBean identified through specified object name.
-     * 
+     *
      * @param objectName The MBean's object name
      * @throws ManagementException If unregistering the MBean fails
      */
@@ -302,7 +324,7 @@ public abstract class AbstractAgent {
 
     /**
      * Adds the MBean identified by given object name to {@link GaugeMonitor} for being observed
-     * 
+     *
      * @param objectName The MBean's object name
      * @param attributeName The observed attribute
      * @throws ManagementException If adding the MBean to {@link GaugeMonitor} fails
@@ -343,7 +365,7 @@ public abstract class AbstractAgent {
 
     /**
      * Removes the observed MBean identified by specified object name from the {@link GaugeMonitor}
-     * 
+     *
      * @param objectName The MBean's object name
      * @throws ManagementException If removing the MBean from {@link GaugeMonitor} fails
      */
@@ -356,17 +378,17 @@ public abstract class AbstractAgent {
 
     /**
      * Adds a RMI registry
-     * 
+     *
      * @param port The port on which the RMI registry should listen on
      * @param bindAddr The bind address for created sockets by RMI registry
      * @throws ManagementException If RMI registry cannot be created
      */
     protected final void addRMIRegistry(final int port, final String bindAddr) throws ManagementException {
-        Registry registry = null;
         try {
             if (initialized.compareAndSet(false, true)) {
                 rmiSocketFactory = new AbstractAgentSocketFactory(0, bindAddr);
             }
+            Registry registry = null;
             try {
                 /*
                  * If following calls succeed, a RMI registry has already been created that listens on this port
@@ -380,7 +402,7 @@ public abstract class AbstractAgent {
                 /*
                  * Create a new one
                  */
-                registry = LocateRegistry.createRegistry(port, rmiSocketFactory, rmiSocketFactory);
+                registry = LocateRegistry.createRegistry(port, null, rmiSocketFactory);
             }
             registries.put(Integer.valueOf(port), registry);
             if (LOG.isInfoEnabled()) {
@@ -396,7 +418,7 @@ public abstract class AbstractAgent {
 
     /**
      * Lists the RMI registries ports
-     * 
+     *
      * @return An array of <code>int</code> representing RMI ports
      */
     public final int[] getRMIRegistryPorts() {
@@ -411,7 +433,7 @@ public abstract class AbstractAgent {
 
     /**
      * Creates a JMX connector server bound to specified JMX URL.
-     * 
+     *
      * @param urlstr The JMX URL as a string
      * @param jmxLogin The JMX login or <code>null</code> to use no authentication for connecting to specified JMX URL
      * @param jmxPassword The JMX password (only needed if previous parameter is not <code>null</code>)
@@ -419,14 +441,25 @@ public abstract class AbstractAgent {
      * @throws ManagementException If connector cannot be added
      */
     protected final JMXServiceURL addConnectorServer(final String urlstr, final String jmxLogin, final String jmxPassword) throws ManagementException {
-        final JMXServiceURL url;
         try {
-            url = new JMXServiceURL(urlstr);
+            return addConnectorServer(new JMXServiceURL(urlstr), jmxLogin, jmxPassword);
         } catch (final MalformedURLException e) {
             throw new ManagementException(ManagementException.Code.MALFORMED_URL, e, urlstr);
         }
+    }
+
+    /**
+     * Creates a JMX connector server bound to specified JMX URL.
+     *
+     * @param url The JMX URL
+     * @param jmxLogin The JMX login or <code>null</code> to use no authentication for connecting to specified JMX URL
+     * @param jmxPassword The JMX password (only needed if previous parameter is not <code>null</code>)
+     * @return The {@link JMXServiceURL} to which the connector is bound
+     * @throws ManagementException If connector cannot be added
+     */
+    protected final JMXServiceURL addConnectorServer(final JMXServiceURL url, final String jmxLogin, final String jmxPassword) throws ManagementException {
         if (connectors.containsKey(url)) {
-            throw new ManagementException(ManagementException.Code.JMX_URL_ALREADY_BOUND, urlstr);
+            throw new ManagementException(ManagementException.Code.JMX_URL_ALREADY_BOUND, url);
         }
         try {
             final Map<String, Object> environment;
@@ -444,7 +477,7 @@ public abstract class AbstractAgent {
             cs.start();
             connectors.put(url, cs);
             if (LOG.isInfoEnabled()) {
-                LOG.info(new StringBuilder("JMX connector server on ").append(urlstr).append(" started"));
+                LOG.info(new StringBuilder("JMX connector server on ").append(url).append(" started"));
             }
             return url;
         } catch (final IOException e) {
@@ -454,7 +487,7 @@ public abstract class AbstractAgent {
 
     /**
      * Removes the JMX connector server bound to specified JMX URL
-     * 
+     *
      * @param url The JMX URL
      */
     protected final void removeConnectorServer(final JMXServiceURL url) {
@@ -495,7 +528,7 @@ public abstract class AbstractAgent {
      * <li>If any exceptional condition is experienced, <code> 0 </code> is returned, indicating that the method failed for some reasons and
      * the callers should take the corrective action. (The method need not always throw an exception for this).
      * <li>Method is synchronized on this class.
-     * 
+     *
      * @return integer depicting the free port number available at this time 0 otherwise.
      */
     protected static final int getFreePort() {
